@@ -1,4 +1,4 @@
-use std::{collections::HashMap, path::PathBuf};
+use std::collections::HashMap;
 
 use layeredconf::{Builder, Format, LayeredConf, Source};
 use serde::{Deserialize, Serialize};
@@ -38,22 +38,22 @@ struct Database {
 
 fn main() -> anyhow::Result<()> {
     let mut builder = Builder::<Config>::new();
-    builder.new_layer(Source::File {
-        path: PathBuf::from("examples/layers/lowest.yaml"),
-        format: Format::Auto,
-    });
-    builder.new_layer(Source::File {
-        path: PathBuf::from("examples/layers/mid.toml"),
-        format: Format::Auto,
-    });
-    builder.new_layer(Source::File {
-        path: PathBuf::from("examples/layers/highest.json"),
-        format: Format::Auto,
-    });
-    builder.new_layer(Source::FileOptional {
-        path: PathBuf::from("examples/layers/does_not_exist.json"),
-        format: Format::Auto,
-    });
+    builder.new_layer(Source::File(
+        "examples/layers/lowest.yaml".parse()?,
+        Format::Auto,
+    ));
+    builder.new_layer(Source::File(
+        "examples/layers/mid.toml".parse()?,
+        Format::Auto,
+    ));
+    builder.new_layer(Source::File(
+        "examples/layers/highest.json".parse()?,
+        Format::Auto,
+    ));
+    builder.new_layer(Source::FileOptional(
+        "examples/layers/does_not_exist.json".parse()?,
+        Format::Auto,
+    ));
     builder.new_layer(Source::Arguments);
     let solid = builder.solidify()?;
 
