@@ -2,11 +2,9 @@
 
 use std::{
     collections::HashSet,
+    env::current_dir,
     path::{Path, PathBuf},
-    sync::{
-        atomic::{AtomicBool, Ordering},
-        Arc, Mutex,
-    },
+    sync::{Arc, Mutex},
 };
 
 use clap::Parser;
@@ -29,7 +27,7 @@ use super::{LayeredConfLayer, LayeredConfMerge, LayeredConfSolid, LayeredConfSol
 ///
 /// fn main() -> anyhow::Result<()> {
 ///     let config: Config = Builder::new()
-///         .new_layer(Source::String { str: "{ \"name\": \"test\" }".to_string(), format: Format::Json })
+///         .new_layer(Source::String("name: test".to_string(), Format::Yaml))
 ///         .solidify()?;
 ///
 ///     Ok(())
@@ -74,28 +72,18 @@ where
 
     /// Adds a new Layer to the Builder from a source
     pub fn new_layer(&mut self, source: Source) -> &mut Self {
-        let layer = Arc::from(Layer::new(source));
+        let layer = Arc::from(Layer::new(source, None, vec![]));
         self.layers.push(layer);
         self
     }
 
-    /// Loads all the config sources defined in the builder
-    pub fn load_all(&self) -> Result<()> {
-        for layer in self.layers.iter() {
-            layer.load()?;
-        }
-        Ok(())
-    }
-
     /// Solidifies the Builder ingo a Config
     pub fn solidify(&self) -> Result<TSolid> {
         if self.layers.is_empty() {
             return Err(Error::SolidifyFailedNoLayers);
         }
         for layer in &self.layers {
-            if !layer.loaded.load(Ordering::Relaxed) {
-                layer.load()?;
-            }
+            layer.load()?;
         }
 
         let mut merged = <TSolid>::Layer::default();
@@ -145,9 +133,10 @@ where
         + Sized,
 {
     source: Source,
+    cwd: Option<PathBuf>,
+    parents: Vec<Source>,
     obj: Mutex<<TSolid>::Layer>,
     sub_layers: Mutex<Vec<Layer<TSolid>>>,
-    loaded: AtomicBool,
 }
 
 impl<TSolid> Layer<TSolid>
@@ -164,12 +153,13 @@ where
         + clap::Parser
         + Sized,
 {
-    fn new(source: Source) -> Self {
+    fn new(source: Source, cwd: Option<PathBuf>, parents: Vec<Source>) -> Self {
         Self {
             source,
+            cwd,
+            parents,
             obj: Mutex::from(<TSolid>::Layer::default()),
             sub_layers: Mutex::from(Vec::new()),
-            loaded: AtomicBool::new(false),
         }
     }
 
@@ -184,77 +174,47 @@ where
         Ok(())
     }
 
+    pub fn load(&self) -> super::Result<()> {
+        let mut seen_paths = HashSet::new();
+        self.load_impl(&mut seen_paths)
+    }
+
     fn load_impl(&self, seen_paths: &mut HashSet<PathBuf>) -> super::Result<()> {
         let mut obj = self.obj.lock().unwrap();
         let mut sub_layers = self.sub_layers.lock().unwrap();
 
         *obj = match &self.source {
-            Source::File { path, format } => {
-                let path = self
-                    .canonicalize(path)
-                    .map_err(map_canonicalization_error(path))?;
-
-                if seen_paths.contains(&path) {
-                    return Err(Error::LoopingLoadConfig);
-                }
-
-                let string = std::fs::read_to_string(&path).map_err(map_io_error(&path))?;
-
-                match self.auto_format(&path, format)? {
-                    Format::Auto => return Err(Error::AutoFormatFailed),
-                    Format::Json => serde_json::from_str(&string)?,
-                    Format::Toml => toml::from_str(&string)?,
-                    Format::Yaml => serde_yaml::from_str(&string)?,
-                }
-            }
-            Source::FileOptional { path, format } => match self.canonicalize(path) {
-                Err(error) => match error.kind() {
-                    std::io::ErrorKind::NotFound => <TSolid>::Layer::default(),
-                    _ => {
-                        return Err(Error::IoError {
-                            wrapped: error,
-                            path: path.to_path_buf(),
-                        })
-                    }
-                },
-                Ok(path) => {
-                    if seen_paths.contains(&path) {
-                        return Err(Error::LoopingLoadConfig);
-                    }
-
-                    match std::fs::read_to_string(&path) {
-                        Ok(string) => match self.auto_format(&path, format)? {
-                            Format::Auto => return Err(Error::AutoFormatFailed),
-                            Format::Json => serde_json::from_str(&string)?,
-                            Format::Toml => toml::from_str(&string)?,
-                            Format::Yaml => serde_yaml::from_str(&string)?,
-                        },
-                        Err(_) => <TSolid>::Layer::default(),
-                    }
+            Source::File(path, format) => self.load_file(path, format, seen_paths)?,
+            Source::FileOptional(path, format) => match self.load_file(path, format, seen_paths) {
+                Err(Error::FileNotFound { .. }) => <TSolid>::Layer::default(),
+                Err(error) => {
+                    return Err(error);
                 }
+                Ok(value) => value,
             },
-            Source::String { str, format } => match format {
-                Format::Auto => return Err(Error::AutoFormatFailed),
-                Format::Json => serde_json::from_str(str)?,
-                Format::Toml => toml::from_str(str)?,
-                Format::Yaml => serde_yaml::from_str(str)?,
-            },
-            Source::Environment { prefix: _ } => {
+            Source::String(string, format) => self.load_string(string, format)?,
+            Source::Environment(_) => {
                 unimplemented!();
             }
             Source::Arguments => <TSolid>::Layer::parse(),
             Source::ArgumentsFrom(from) => <TSolid>::Layer::parse_from(from),
         };
 
+        let source_dir = self.get_source_dir()?;
+
         *sub_layers = obj
             .load_configs()
             .iter()
             .cloned()
             .map(|path| {
-                Layer::new(Source::File {
-                    path,
-                    format: Format::Auto,
-                })
+                let mut parents = self.parents.clone();
+                parents.insert(0, self.source.clone());
+
+                Layer::new(
+                    Source::File(path, Format::Auto),
+                    Some(source_dir.clone()),
+                    parents,
+                )
             })
             .collect();
 
@@ -262,13 +222,71 @@ where
             sub_layer.load_impl(seen_paths)?;
         }
 
-        self.loaded.store(true, Ordering::Relaxed);
-
         Ok(())
     }
 
-    fn canonicalize(&self, path: &Path) -> std::io::Result<PathBuf> {
-        std::fs::canonicalize(path)
+    fn get_source_dir(&self) -> Result<PathBuf> {
+        use Source::{File, FileOptional};
+
+        Ok(match &self.source {
+            File(path, _) | FileOptional(path, _) => {
+                let real_path = if path.is_absolute() {
+                    path.to_path_buf()
+                } else {
+                    self.get_cwd()?.join(path)
+                };
+                real_path
+                    .parent()
+                    .ok_or_else(|| Error::ParentDir {
+                        path: real_path.clone(),
+                    })?
+                    .to_path_buf()
+            }
+            _ => self.get_cwd()?,
+        })
+    }
+
+    fn load_file(
+        &self,
+        path: &Path,
+        format: &Format,
+        seen_paths: &mut HashSet<PathBuf>,
+    ) -> Result<<TSolid>::Layer> {
+        let path = if path.is_absolute() {
+            path.to_path_buf()
+        } else {
+            self.get_cwd()?.join(path)
+        };
+        let path = path
+            .canonicalize()
+            .map_err(map_canonicalization_error(&path))?;
+
+        if seen_paths.contains(&path) {
+            let parents = self.parents.clone();
+            return Err(Error::LoopingLoadConfig { parents, path });
+        }
+        seen_paths.insert(path.clone());
+
+        let string = std::fs::read_to_string(&path).map_err(map_io_error(&path))?;
+
+        self.load_string(&string, &self.auto_format(&path, format)?)
+    }
+
+    fn get_cwd(&self) -> Result<PathBuf> {
+        self.cwd
+            .clone()
+            .map(Ok)
+            .unwrap_or_else(current_dir)
+            .map_err(|wrapped| Error::CurrentDir { wrapped })
+    }
+
+    fn load_string(&self, string: &str, format: &Format) -> Result<<TSolid>::Layer> {
+        Ok(match format {
+            Format::Auto => return Err(Error::AutoFormatFailed),
+            Format::Json => serde_json::from_str(string)?,
+            Format::Toml => toml::from_str(string)?,
+            Format::Yaml => serde_yaml::from_str(string)?,
+        })
     }
 
     fn auto_format(&self, path: &Path, format: &Format) -> Result<Format> {
@@ -287,15 +305,10 @@ where
             format => Ok(*format),
         }
     }
-
-    pub fn load(&self) -> super::Result<()> {
-        let mut seen_paths = HashSet::new();
-        self.load_impl(&mut seen_paths)
-    }
 }
 
 /// Config file format
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Format {
     /// Automatically detect config file format from it's extension
     Auto,
@@ -308,34 +321,16 @@ pub enum Format {
 }
 
 /// Config source
-#[derive(Debug)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Source {
     /// From a file
-    File {
-        /// File path
-        path: PathBuf,
-        /// File format
-        format: Format,
-    },
+    File(PathBuf, Format),
     /// From a file, ignoring if it doesn't exist
-    FileOptional {
-        /// File path
-        path: PathBuf,
-        /// File format
-        format: Format,
-    },
+    FileOptional(PathBuf, Format),
     /// From a String
-    String {
-        /// String contents
-        str: String,
-        /// Format
-        format: Format,
-    },
+    String(String, Format),
     /// From process env (currently unimplemented)
-    Environment {
-        /// Optional prefix for environment variables
-        prefix: Option<String>,
-    },
+    Environment(Option<String>),
     /// From argv
     Arguments,
     /// From an Vec of arguments
